@@ -7,7 +7,6 @@
 # SPDX-FileCopyrightText: 2021 Adam Cummick
 #
 # SPDX-License-Identifier: MIT
-
 """
 `adafruit_wiznet5k`
 ================================================================================
@@ -27,6 +26,23 @@ Implementation Notes
 
 * Adafruit's Bus Device library: https://github.com/adafruit/Adafruit_CircuitPython_BusDevice
 """
+
+# pylint: disable=too-many-lines
+from __future__ import annotations
+
+try:
+    from typing import TYPE_CHECKING, Optional, Union, List, Tuple, Sequence
+
+    if TYPE_CHECKING:
+        from circuitpython_typing import WriteableBuffer
+        import busio
+        import digitalio
+except ImportError:
+    pass
+
+__version__ = "1.12.16"
+__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_Wiznet5k.git"
+
 from random import randint
 import time
 from micropython import const
@@ -35,11 +51,6 @@ from adafruit_bus_device.spi_device import SPIDevice
 import adafruit_wiznet5k.adafruit_wiznet5k_dhcp as dhcp
 import adafruit_wiznet5k.adafruit_wiznet5k_dns as dns
 
-
-__version__ = "1.12.14"
-__repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_Wiznet5k.git"
-
-
 # Wiznet5k Registers
 REG_MR = const(0x0000)  # Mode
 REG_GAR = const(0x0001)  # Gateway IP Address
@@ -122,23 +133,12 @@ W5200_W5500_MAX_SOCK_NUM = const(0x08)
 W5100_MAX_SOCK_NUM = const(0x04)
 SOCKET_INVALID = const(255)
 
-
 # Source ports in use
 SRC_PORTS = [0] * W5200_W5500_MAX_SOCK_NUM
 
 
 class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-attributes
-    """Interface for WIZNET5K module.
-
-    :param ~busio.SPI spi_bus: The SPI bus the Wiznet module is connected to.
-    :param ~digitalio.DigitalInOut cs: Chip select pin.
-    :param ~digitalio.DigitalInOut rst: Optional reset pin.
-    :param bool is_dhcp: Whether to start DHCP automatically or not.
-    :param list mac: The Wiznet's MAC Address.
-    :param str hostname: The desired hostname, with optional {} to fill in MAC.
-    :param int dhcp_timeout: Timeout in seconds for DHCP response.
-    :param bool debug: Enable debugging output.
-    """
+    """Interface for WIZNET5K module."""
 
     TCP_MODE = const(0x21)
     UDP_MODE = const(0x02)
@@ -147,15 +147,27 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
     # pylint: disable=too-many-arguments
     def __init__(
         self,
-        spi_bus,
-        cs,
-        reset=None,
-        is_dhcp=True,
-        mac=DEFAULT_MAC,
-        hostname=None,
-        dhcp_timeout=30,
-        debug=False,
-    ):
+        spi_bus: busio.SPI,
+        cs: digitalio.DigitalInOut,  # pylint: disable=invalid-name
+        reset: Optional[digitalio.DigitalInOut] = None,
+        is_dhcp: bool = True,
+        mac: Union[List[int], Tuple[int]] = DEFAULT_MAC,
+        hostname: Optional[str] = None,
+        dhcp_timeout: float = 30.0,
+        debug: bool = False,
+    ) -> None:
+        """
+        :param busio.SPI spi_bus: The SPI bus the Wiznet module is connected to.
+        :param digitalio.DigitalInOut cs: Chip select pin.
+        :param digitalio.DigitalInOut reset: Optional reset pin, defaults to None.
+        :param bool is_dhcp: Whether to start DHCP automatically or not, defaults to True.
+        :param Union[List[int], Tuple[int]] mac: The Wiznet's MAC Address, defaults to
+            (0xDE, 0xAD, 0xBE, 0xEF, 0xFE, 0xED).
+        :param str hostname: The desired hostname, with optional {} to fill in the MAC
+            address, defaults to None.
+        :param float dhcp_timeout: Timeout in seconds for DHCP response, defaults to 30.0.
+        :param bool debug: Enable debugging output, defaults to False.
+        """
         self._debug = debug
         self._chip_type = None
         self._device = SPIDevice(spi_bus, cs, baudrate=8000000, polarity=0, phase=0)
@@ -204,13 +216,19 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
                 self._dhcp_client = None
             assert ret == 0, "Failed to configure DHCP Server!"
 
-    def set_dhcp(self, hostname=None, response_timeout=30):
-        """Initializes the DHCP client and attempts to retrieve
-        and set network configuration from the DHCP server.
-        Returns 0 if DHCP configured, -1 otherwise.
+    def set_dhcp(
+        self, hostname: Optional[str] = None, response_timeout: float = 30
+    ) -> int:
+        """
+        Initialize the DHCP client and attempt to retrieve and set network
+        configuration from the DHCP server.
 
-        :param str hostname: The desired hostname, with optional {} to fill in MAC.
-        :param int response_timeout: Time to wait for server to return packet, in seconds.
+        :param Optional[str] hostname: The desired hostname for the DHCP server with optional {} to
+            fill in the MAC address, defaults to None.
+        :param float response_timeout: Time to wait for server to return packet in seconds,
+            defaults to 30.0.
+
+        :return int: 0 if DHCP configured, -1 otherwise.
         """
         if self._debug:
             print("* Initializing DHCP")
@@ -232,14 +250,18 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             return 0
         return -1
 
-    def maintain_dhcp_lease(self):
-        """Maintain DHCP lease"""
+    def maintain_dhcp_lease(self) -> None:
+        """Maintain the DHCP lease."""
         if self._dhcp_client is not None:
             self._dhcp_client.maintain_dhcp_lease()
 
-    def get_host_by_name(self, hostname):
-        """Convert a hostname to a packed 4-byte IP Address.
-        Returns a 4 bytearray.
+    def get_host_by_name(self, hostname: str) -> bytes:
+        """
+        Convert a hostname to a packed 4-byte IP Address.
+
+        :param str hostname: The host name to be converted.
+
+        :return Union[int, bytes]: a 4 bytearray.
         """
         if self._debug:
             print("* Get host by name")
@@ -254,8 +276,12 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
         return ret
 
     @property
-    def max_sockets(self):
-        """Returns max number of sockets supported by chip."""
+    def max_sockets(self) -> int:
+        """
+        Maximum number of sockets supported by chip.
+
+        :return int: Maximum supported sockets.
+        """
         if self._chip_type == "w5500":
             return W5200_W5500_MAX_SOCK_NUM
         if self._chip_type == "w5100s":
@@ -263,44 +289,80 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
         return -1
 
     @property
-    def chip(self):
-        """Returns the chip type."""
+    def chip(self) -> str:
+        """
+        Ethernet controller chip type.
+
+        :return str: The chip type.
+        """
         return self._chip_type
 
     @property
-    def ip_address(self):
-        """Returns the configured IP address."""
+    def ip_address(self) -> bytearray:
+        """
+        Configured IP address.
+
+        :return bytearray: IP address as four bytes.
+        """
         return self.read(REG_SIPR, 0x00, 4)
 
-    def pretty_ip(self, ip):  # pylint: disable=no-self-use, invalid-name
-        """Converts a bytearray IP address to a
-        dotted-quad string for printing
+    def pretty_ip(
+        self,
+        # pylint: disable=no-self-use, invalid-name
+        ip: bytearray,
+    ) -> str:
+        """
+        Convert a 4 byte IP address to a dotted-quad string for printing.
 
+        :param bytearray ip: A four byte IP address.
+
+        :return str: The IP address (a string of the form '255.255.255.255').
         """
         return "%d.%d.%d.%d" % (ip[0], ip[1], ip[2], ip[3])
 
-    def unpretty_ip(self, ip):  # pylint: disable=no-self-use, invalid-name
-        """Converts a dotted-quad string to a bytearray IP address"""
+    def unpretty_ip(
+        self,
+        # pylint: disable=no-self-use, invalid-name
+        ip: str,
+    ) -> bytes:
+        """
+        Convert a dotted-quad string to a four byte IP address.
+
+        :param str ip: IP address (a string of the form '255.255.255.255') to be converted.
+
+        :return bytes: IP address in four bytes.
+        """
         octets = [int(x) for x in ip.split(".")]
         return bytes(octets)
 
     @property
-    def mac_address(self):
-        """Returns the hardware's MAC address."""
+    def mac_address(self) -> bytearray:
+        """
+        Ethernet hardware's MAC address.
+
+        :return bytearray: Six byte MAC address."""
         return self.read(REG_SHAR, 0x00, 6)
 
     @mac_address.setter
-    def mac_address(self, address):
-        """Sets the hardware MAC address.
+    def mac_address(self, address: Sequence[Union[int, bytes]]) -> None:
+        """
+        Sets the hardware MAC address.
 
         :param tuple address: Hardware MAC address.
         """
         self.write(REG_SHAR, 0x04, address)
 
-    def pretty_mac(self, mac):  # pylint: disable=no-self-use, invalid-name
-        """Converts a bytearray MAC address to a
-        dotted-quad string for printing
+    def pretty_mac(
+        self,
+        # pylint: disable=no-self-use, invalid-name
+        mac: bytearray,
+    ) -> str:
+        """
+        Convert a bytearray MAC address to a ':' seperated string for display.
 
+        :param bytearray mac: The MAC address.
+
+        :return str: Mac Address in the form 00:00:00:00:00:00
         """
         return "%s:%s:%s:%s:%s:%s" % (
             hex(mac[0]),
@@ -311,10 +373,13 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             hex(mac[5]),
         )
 
-    def remote_ip(self, socket_num):
-        """Returns the IP address of the host who sent the current incoming packet.
+    def remote_ip(self, socket_num: int) -> Union[str, bytearray]:
+        """
+        IP address of the host which sent the current incoming packet.
 
-        :param int socket num: Desired socket.
+        :param int socket_num: ID number of the socket to check.
+
+        :return Union[str, bytearray]: A four byte IP address.
         """
         if socket_num >= self.max_sockets:
             return self._pbuff
@@ -323,8 +388,11 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
         return self.pretty_ip(self._pbuff)
 
     @property
-    def link_status(self):
-        """ "Returns if the PHY is connected."""
+    def link_status(self) -> int:
+        """Physical hardware (PHY) connection status.
+
+        :return int: 1 if the link is up, 0 if the link is down.
+        """
         if self._chip_type == "w5500":
             data = self.read(REG_PHYCFGR, 0x00)
             return data[0] & 0x01
@@ -333,17 +401,29 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             return data[0] & 0x01
         return 0
 
-    def remote_port(self, socket_num):
-        """Returns the port of the host who sent the current incoming packet."""
+    def remote_port(self, socket_num: int) -> Union[int, bytearray]:
+        """
+        Port of the host which sent the current incoming packet.
+
+        :param int socket_num: ID number of the socket to check.
+
+        :return Union[int, bytearray]: The port number of the socket connection.
+        """
         if socket_num >= self.max_sockets:
             return self._pbuff
-        for octet in range(0, 2):
+        for octet in range(2):
             self._pbuff[octet] = self._read_socket(socket_num, REG_SNDPORT + octet)[0]
         return int((self._pbuff[0] << 8) | self._pbuff[0])
 
     @property
-    def ifconfig(self):
-        """Returns the network configuration as a tuple."""
+    def ifconfig(
+        self,
+    ) -> Tuple[bytearray, bytearray, bytearray, Tuple[int, int, int, int]]:
+        """
+        Network configuration information.
+
+        :return Tuple[bytearray, bytearray, bytearray, Tuple[int, int, int, int]]: \
+            The IP address, subnet mask, gateway address and DNS server address."""
         return (
             self.ip_address,
             self.read(REG_SUBR, 0x00, 4),
@@ -352,10 +432,14 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
         )
 
     @ifconfig.setter
-    def ifconfig(self, params):
-        """Sets network configuration to provided tuple in format:
-        (ip_address, subnet_mask, gateway_address, dns_server).
+    def ifconfig(
+        self, params: Tuple[bytearray, bytearray, bytearray, Tuple[int, int, int, int]]
+    ) -> None:
+        """
+        Set network configuration.
 
+        :param Tuple[bytearray, bytearray, bytearray, Tuple[int, int, int, int]]:
+            Configuration settings - (ip_address, subnet_mask, gateway_address, dns_server).
         """
         ip_address, subnet_mask, gateway_address, dns_server = params
 
@@ -365,8 +449,12 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
 
         self._dns = dns_server
 
-    def _w5100_init(self):
-        """Initializes and detects a wiznet5k module."""
+    def _w5100_init(self) -> int:
+        """
+        Detect and initialize a Wiznet5k ethernet module.
+
+        :return int: 1 if the initialization succeeds, 0 if it fails.
+        """
         time.sleep(1)
         self._cs.switch_to_output()
         self._cs.value = 1
@@ -386,8 +474,12 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
                 return 0
         return 1
 
-    def detect_w5500(self):
-        """Detects W5500 chip."""
+    def detect_w5500(self) -> int:
+        """
+        Detect W5500 chip.
+
+        :return int: 1 if a W5500 chip is detected, -1 if not.
+        """
         self._chip_type = "w5500"
         assert self.sw_reset() == 0, "Chip not reset properly!"
         self._write_mr(0x08)
@@ -411,8 +503,12 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
         # self._ch_base_msb = 0x10
         return 1
 
-    def detect_w5100s(self):
-        """Detects W5100S chip."""
+    def detect_w5100s(self) -> int:
+        """
+        Detect W5100S chip.
+
+        :return int: 1 if a W5100 chip is detected, -1 if not.
+        """
         self._chip_type = "w5100s"
         # sw reset
         assert self.sw_reset() == 0, "Chip not reset properly!"
@@ -422,10 +518,12 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
         self._ch_base_msb = 0x0400
         return 1
 
-    def sw_reset(self):
-        """Performs a soft-reset on a Wiznet chip
-        by writing to its MR register reset bit.
+    def sw_reset(self) -> int:
+        """Perform a soft-reset on the Wiznet chip.
 
+        Perform a soft reset by writing to the chip's MR register reset bit.
+
+        :return int: 0 if the reset succeeds, -1 if not.
         """
         mode_reg = self._read_mr()
         self._write_mr(0x80)
@@ -436,22 +534,31 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             return -1
         return 0
 
-    def _read_mr(self):
-        """Reads from the Mode Register (MR)."""
+    def _read_mr(self) -> bytearray:
+        """Read from the Mode Register (MR)."""
         res = self.read(REG_MR, 0x00)
         return res
 
-    def _write_mr(self, data):
-        """Writes to the mode register (MR).
-
-        :param int data: Data to write to the mode register.
-        """
+    def _write_mr(self, data: int) -> None:
+        """Write to the mode register (MR)."""
         self.write(REG_MR, 0x04, data)
 
-    def read(self, addr, callback, length=1, buffer=None):
-        """Reads data from a register address.
+    def read(
+        self,
+        addr: int,
+        callback: int,
+        length: int = 1,
+        buffer: Optional[WriteableBuffer] = None,
+    ) -> Union[WriteableBuffer, bytearray]:
+        """
+        Read data from a register address.
 
-        :param int addr: Register address.
+        :param int addr: Register address to read.
+        :param int callback: Callback reference.
+        :param int length: Number of bytes to read from the register, defaults to 1.
+        :param Optional[WriteableBuffer] buffer: Buffer to read data into, defaults to None.
+
+        :return Union[WriteableBuffer, bytearray]: Data read from the chip.
         """
         with self._device as bus_device:
             if self._chip_type == "w5500":
@@ -471,13 +578,15 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             bus_device.readinto(buffer, end=length)  # pylint: disable=no-member
             return buffer
 
-    def write(self, addr, callback, data):
-        """Write data to a register address.
+    def write(
+        self, addr: int, callback: int, data: Union[int, Sequence[Union[int, bytes]]]
+    ) -> None:
+        """
+        Write data to a register address.
 
         :param int addr: Destination address.
         :param int callback: Callback reference.
-        :param int data: Data to write, as an integer.
-        :param bytearray data: Data to write, as a bytearray.
+        :param Union[int, Sequence[Union[int, bytes]]] data: Data to write to the register address.
         """
         with self._device as bus_device:
             if self._chip_type == "w5500":
@@ -493,16 +602,20 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             if hasattr(data, "from_bytes"):
                 bus_device.write(bytes([data]))  # pylint: disable=no-member
             else:
-                for i, _ in enumerate(data):
-                    bus_device.write(bytes([data[i]]))  # pylint: disable=no-member
+                for data_comp in data:
+                    bus_device.write(bytes([data_comp]))  # pylint: disable=no-member
 
     # Socket-Register API
 
-    def socket_available(self, socket_num, sock_type=SNMR_TCP):
-        """Returns the amount of bytes to be read from the socket.
+    def socket_available(self, socket_num: int, sock_type: int = SNMR_TCP) -> int:
+        """
+        Number of bytes available to be read from the socket.
 
-        :param int socket_num: Desired socket to return bytes from.
-        :param int sock_type: Socket type, defaults to TCP.
+        :param int socket_num: Socket to check for available bytes.
+        :param int sock_type: Socket type. Use SNMR_TCP for TCP or SNMR_UDP for UDP, \
+            defaults to SNMR_TCP.
+
+        :return int: Number of bytes available to read.
         """
         if self._debug:
             print(
@@ -530,19 +643,38 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
                 return ret
         return 0
 
-    def socket_status(self, socket_num):
-        """Returns the socket connection status. Can be: SNSR_SOCK_CLOSED,
-        SNSR_SOCK_INIT, SNSR_SOCK_LISTEN, SNSR_SOCK_SYNSENT, SNSR_SOCK_SYNRECV,
-        SNSR_SYN_SOCK_ESTABLISHED, SNSR_SOCK_FIN_WAIT, SNSR_SOCK_CLOSING,
-        SNSR_SOCK_TIME_WAIT, SNSR_SOCK_CLOSE_WAIT, SNSR_LAST_ACK,
+    def socket_status(self, socket_num: int) -> Optional[bytearray]:
+        """
+        Socket connection status.
+
+        Can be: SNSR_SOCK_CLOSED, SNSR_SOCK_INIT, SNSR_SOCK_LISTEN, SNSR_SOCK_SYNSENT,
+        SNSR_SOCK_SYNRECV, SNSR_SYN_SOCK_ESTABLISHED, SNSR_SOCK_FIN_WAIT,
+        SNSR_SOCK_CLOSING, SNSR_SOCK_TIME_WAIT, SNSR_SOCK_CLOSE_WAIT, SNSR_LAST_ACK,
         SNSR_SOCK_UDP, SNSR_SOCK_IPRAW, SNSR_SOCK_MACRAW, SNSR_SOCK_PPOE.
+
+        :param int socket_num: ID of socket to check.
+
+        :return: Optional[bytearray]
         """
         return self._read_snsr(socket_num)
 
-    def socket_connect(self, socket_num, dest, port, conn_mode=SNMR_TCP):
-        """Open and verify we've connected a socket to a dest IP address
-        or hostname. By default, we use 'conn_mode'= SNMR_TCP but we
-        may also use SNMR_UDP.
+    def socket_connect(
+        self,
+        socket_num: int,
+        dest: Union[bytes, bytearray],
+        port: int,
+        conn_mode: int = SNMR_TCP,
+    ) -> int:
+        """
+        Open and verify a connection from a socket to a destination IP address
+        or hostname. A TCP connection is made by default. A UDP connection can also
+        be made.
+
+        :param int socket_num: ID of the socket to be connected.
+        :param Union[bytes, bytearray] dest: The destination as a host name or IP address.
+        :param int port: Port to connect to (0 - 65,536).
+        :param int conn_mode: The connection mode. Use SNMR_TCP for TCP or SNMR_UDP for UDP,
+            defaults to SNMR_TCP.
         """
         assert self.link_status, "Ethernet cable disconnected!"
         if self._debug:
@@ -554,7 +686,7 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
         # initialize a socket and set the mode
         res = self.socket_open(socket_num, conn_mode=conn_mode)
         if res == 1:
-            raise RuntimeError("Failed to initalize a connection with the socket.")
+            raise RuntimeError("Failed to initialize a connection with the socket.")
 
         # set socket destination IP and port
         self._write_sndipr(socket_num, dest)
@@ -573,15 +705,19 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             self.udp_datasize[socket_num] = 0
         return 1
 
-    def _send_socket_cmd(self, socket, cmd):
+    def _send_socket_cmd(self, socket: int, cmd: int) -> None:
+        """Send a socket command to a socket."""
         self._write_sncr(socket, cmd)
         while self._read_sncr(socket) != b"\x00":
             if self._debug:
                 print("waiting for sncr to clear...")
 
-    def get_socket(self):
-        """Requests, allocates and returns a socket from the W5k
-        chip. Returned socket number may not exceed max_sockets.
+    def get_socket(self) -> int:
+        """Request, allocate and return a socket from the W5k chip.
+
+        Cycle through the sockets to find the first available one, if any.
+
+        :return int: The first available socket. Returns 0xFF if no sockets are free.
         """
         if self._debug:
             print("*** Get socket")
@@ -597,12 +733,16 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             print("Allocated socket #{}".format(sock))
         return sock
 
-    def socket_listen(self, socket_num, port, conn_mode=SNMR_TCP):
-        """Start listening on a socket (default TCP mode).
+    def socket_listen(
+        self, socket_num: int, port: int, conn_mode: int = SNMR_TCP
+    ) -> None:
+        """
+        Listen on a socket's port.
 
-        :param int socket_num: socket number
-        :param int port: port to listen on
-        :param int conn_mode: connection mode SNMR_TCP (default) or SNMR_UDP
+        :param int socket_num: ID of socket to listen on.
+        :param int port: Port to listen on (0 - 65,535).
+        :param int conn_mode: Connection mode SNMR_TCP for TCP or SNMR_UDP for
+            UDP, defaults to SNMR_TCP.
         """
         assert self.link_status, "Ethernet cable disconnected!"
         if self._debug:
@@ -616,7 +756,7 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
         res = self.socket_open(socket_num, conn_mode=conn_mode)
         self.src_port = 0
         if res == 1:
-            raise RuntimeError("Failed to initalize the socket.")
+            raise RuntimeError("Failed to initialize the socket.")
         # Send listen command
         self._send_socket_cmd(socket_num, CMD_SOCK_LISTEN)
         # Wait until ready
@@ -626,11 +766,21 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             if status[0] == SNSR_SOCK_CLOSED:
                 raise RuntimeError("Listening socket closed.")
 
-    def socket_accept(self, socket_num):
-        """Gets the dest IP and port from an incoming connection.
-        Returns the next socket number so listening can continue
+    def socket_accept(
+        self, socket_num: int
+    ) -> Tuple[int, Tuple[Union[str, bytearray], Union[int, bytearray]]]:
+        """
+        Destination IP address and port from an incoming connection.
 
-        :param int socket_num: socket number
+        Return the next socket number so listening can continue, along with
+        the IP address and port of the incoming connection.
+
+        :param int socket_num: Socket number with connection to check.
+        :return Tuple[int, Tuple[Union[str, bytearray], Union[int, bytearray]]]:
+            If successful, the next (socket number, (destination IP address, destination port)).
+
+        If errors occur, the destination IP address and / or the destination port may be
+        returned as bytearrays.
         """
         dest_ip = self.remote_ip(socket_num)
         dest_port = self.remote_port(socket_num)
@@ -643,9 +793,16 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             )
         return next_socknum, (dest_ip, dest_port)
 
-    def socket_open(self, socket_num, conn_mode=SNMR_TCP):
-        """Opens a TCP or UDP socket. By default, we use
-        'conn_mode'=SNMR_TCP but we may also use SNMR_UDP.
+    def socket_open(self, socket_num: int, conn_mode: int = SNMR_TCP) -> int:
+        """
+        Open an IP socket.
+
+        The socket may connect via TCP or UDP protocols.
+
+        :param int socket_num: The socket number to open.
+        :param int conn_mode: The protocol to use. Use SNMR_TCP for TCP or SNMR_UDP for \
+            UDP, defaults to SNMR_TCP.
+        :return int: 1 if the socket was opened, 0 if not.
         """
         assert self.link_status, "Ethernet cable disconnected!"
         if self._debug:
@@ -686,23 +843,41 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             return 0
         return 1
 
-    def socket_close(self, socket_num):
-        """Closes a socket."""
+    def socket_close(self, socket_num: int) -> None:
+        """
+        Close a socket.
+
+        :param int socket_num: The socket to close.
+        """
         if self._debug:
             print("*** Closing socket #%d" % socket_num)
         self._write_sncr(socket_num, CMD_SOCK_CLOSE)
         self._read_sncr(socket_num)
 
-    def socket_disconnect(self, socket_num):
-        """Disconnect a TCP connection."""
+    def socket_disconnect(self, socket_num: int) -> None:
+        """
+        Disconnect a TCP or UDP connection.
+
+        :param int socket_num: The socket to close.
+        """
         if self._debug:
             print("*** Disconnecting socket #%d" % socket_num)
         self._write_sncr(socket_num, CMD_SOCK_DISCON)
         self._read_sncr(socket_num)
 
-    def socket_read(self, socket_num, length):
-        """Reads data from a socket into a buffer.
-        Returns buffer.
+    def socket_read(
+        self, socket_num: int, length: int
+    ) -> Tuple[int, Union[int, bytearray]]:
+        """
+        Read data from a TCP socket.
+
+        :param int socket_num: The socket to read data from.
+        :param int length: The number of bytes to read from the socket.
+
+        :return Tuple[int, Union[int, bytearray]]: If the read was successful then the first
+            item of the tuple is the length of the data and the second is the data. If the read
+            was unsuccessful then both items equal an error code, 0 for no data waiting and -1
+            for no connection to the socket.
         """
         assert self.link_status, "Ethernet cable disconnected!"
         assert socket_num <= self.max_sockets, "Provided socket exceeds max_sockets."
@@ -761,8 +936,19 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             self._read_sncr(socket_num)
         return ret, resp
 
-    def read_udp(self, socket_num, length):
-        """Read UDP socket's current message bytes."""
+    def read_udp(
+        self, socket_num: int, length: int
+    ) -> Union[int, Tuple[int, Union[int, bytearray]]]:
+        """
+        Read UDP socket's current message bytes.
+
+        :param int socket_num: The socket to read data from.
+        :param int length: The number of bytes to read from the socket.
+
+        :return Union[int, Tuple[int, Union[int, bytearray]]]: If the read was successful then
+            the first item of the tuple is the length of the data and the second is the data.
+            If the read was unsuccessful then -1 is returned.
+        """
         if self.udp_datasize[socket_num] > 0:
             if self.udp_datasize[socket_num] <= length:
                 ret, resp = self.socket_read(socket_num, self.udp_datasize[socket_num])
@@ -774,8 +960,19 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             return ret, resp
         return -1
 
-    def socket_write(self, socket_num, buffer, timeout=0):
-        """Writes a bytearray to a provided socket."""
+    def socket_write(
+        self, socket_num: int, buffer: bytearray, timeout: float = 0
+    ) -> int:
+        """
+        Write data to a socket.
+
+        :param int socket_num: The socket to write to.
+        :param bytearray buffer: The data to write to the socket.
+        :param float timeout: Write data timeout in seconds, defaults to 0.0 which waits
+            indefinitely.
+
+        :return int: The number of bytes written to the buffer.
+        """
         assert self.link_status, "Ethernet cable disconnected!"
         assert socket_num <= self.max_sockets, "Provided socket exceeds max_sockets."
         status = 0
@@ -850,9 +1047,8 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
         return ret
 
     # Socket-Register Methods
-
-    def _get_rx_rcv_size(self, sock):
-        """Get size of recieved and saved in socket buffer."""
+    def _get_rx_rcv_size(self, sock: int) -> int:
+        """Size of received and saved in socket buffer."""
         val = 0
         val_1 = self._read_snrx_rsr(sock)
         while val != val_1:
@@ -861,8 +1057,8 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
                 val = self._read_snrx_rsr(sock)
         return int.from_bytes(val, "big")
 
-    def _get_tx_free_size(self, sock):
-        """Get free size of sock's tx buffer block."""
+    def _get_tx_free_size(self, sock: int) -> int:
+        """Free size of socket's tx buffer block."""
         val = 0
         val_1 = self._read_sntx_fsr(sock)
         while val != val_1:
@@ -871,71 +1067,71 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
                 val = self._read_sntx_fsr(sock)
         return int.from_bytes(val, "big")
 
-    def _read_snrx_rd(self, sock):
+    def _read_snrx_rd(self, sock: int) -> int:
         self._pbuff[0] = self._read_socket(sock, REG_SNRX_RD)[0]
         self._pbuff[1] = self._read_socket(sock, REG_SNRX_RD + 1)[0]
         return self._pbuff[0] << 8 | self._pbuff[1]
 
-    def _write_snrx_rd(self, sock, data):
+    def _write_snrx_rd(self, sock: int, data: int) -> None:
         self._write_socket(sock, REG_SNRX_RD, data >> 8 & 0xFF)
         self._write_socket(sock, REG_SNRX_RD + 1, data & 0xFF)
 
-    def _write_sntx_wr(self, sock, data):
+    def _write_sntx_wr(self, sock: int, data: int) -> None:
         self._write_socket(sock, REG_SNTX_WR, data >> 8 & 0xFF)
         self._write_socket(sock, REG_SNTX_WR + 1, data & 0xFF)
 
-    def _read_sntx_wr(self, sock):
+    def _read_sntx_wr(self, sock: int) -> int:
         self._pbuff[0] = self._read_socket(sock, 0x0024)[0]
         self._pbuff[1] = self._read_socket(sock, 0x0024 + 1)[0]
         return self._pbuff[0] << 8 | self._pbuff[1]
 
-    def _read_sntx_fsr(self, sock):
+    def _read_sntx_fsr(self, sock: int) -> Optional[bytearray]:
         data = self._read_socket(sock, REG_SNTX_FSR)
         data += self._read_socket(sock, REG_SNTX_FSR + 1)
         return data
 
-    def _read_snrx_rsr(self, sock):
+    def _read_snrx_rsr(self, sock: int) -> Optional[bytearray]:
         data = self._read_socket(sock, REG_SNRX_RSR)
         data += self._read_socket(sock, REG_SNRX_RSR + 1)
         return data
 
-    def _write_sndipr(self, sock, ip_addr):
-        """Writes to socket destination IP Address."""
+    def _write_sndipr(self, sock: int, ip_addr: bytearray) -> None:
+        """Write to socket destination IP Address."""
         for octet in range(0, 4):
             self._write_socket(sock, REG_SNDIPR + octet, ip_addr[octet])
 
-    def _write_sndport(self, sock, port):
-        """Writes to socket destination port."""
+    def _write_sndport(self, sock: int, port: int) -> None:
+        """Write to socket destination port."""
         self._write_socket(sock, REG_SNDPORT, port >> 8)
         self._write_socket(sock, REG_SNDPORT + 1, port & 0xFF)
 
-    def _read_snsr(self, sock):
-        """Reads Socket n Status Register."""
+    def _read_snsr(self, sock: int) -> Optional[bytearray]:
+        """Read Socket n Status Register."""
         return self._read_socket(sock, REG_SNSR)
 
-    def _write_snmr(self, sock, protocol):
+    def _write_snmr(self, sock: int, protocol: int) -> None:
         """Write to Socket n Mode Register."""
         self._write_socket(sock, REG_SNMR, protocol)
 
-    def _write_snir(self, sock, data):
+    def _write_snir(self, sock: int, data: int) -> None:
         """Write to Socket n Interrupt Register."""
         self._write_socket(sock, REG_SNIR, data)
 
-    def _write_sock_port(self, sock, port):
+    def _write_sock_port(self, sock: int, port: int) -> None:
         """Write to the socket port number."""
         self._write_socket(sock, REG_SNPORT, port >> 8)
         self._write_socket(sock, REG_SNPORT + 1, port & 0xFF)
 
-    def _write_sncr(self, sock, data):
+    def _write_sncr(self, sock: int, data: int) -> None:
         self._write_socket(sock, REG_SNCR, data)
 
-    def _read_sncr(self, sock):
+    def _read_sncr(self, sock: int) -> Optional[bytearray]:
         return self._read_socket(sock, REG_SNCR)
 
-    def _read_snmr(self, sock):
+    def _read_snmr(self, sock: int) -> Optional[bytearray]:
         return self._read_socket(sock, REG_SNMR)
 
-    def _write_socket(self, sock, address, data):
+    def _write_socket(self, sock: int, address: int, data: int) -> None:
         """Write to a W5k socket register."""
         if self._chip_type == "w5500":
             cntl_byte = (sock << 5) + 0x0C
@@ -947,7 +1143,7 @@ class WIZNET5K:  # pylint: disable=too-many-public-methods, too-many-instance-at
             )
         return None
 
-    def _read_socket(self, sock, address):
+    def _read_socket(self, sock: int, address: int) -> Optional[bytearray]:
         """Read a W5k socket register."""
         if self._chip_type == "w5500":
             cntl_byte = (sock << 5) + 0x08
